<?php

namespace Drupal\vite;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Site\Settings;

/**
 * Drupal library vite config.
 */
class AssetLibrary {

  /**
   * Drupal asset library id.
   *
   * @var string
   */
  private $libraryId;

  /**
   * Drupal asset library.
   *
   * @var array
   */
  private $library;

  /**
   * Drupal asset library extension (module/theme) id.
   *
   * @var string
   */
  private $extension;

  /**
   * Constructs AssetLibrary object.
   */
  public function __construct(string $libraryId, array $library, string $extension) {
    $this->libraryId = $libraryId;
    $this->library = $library;
    $this->extension = $extension;
  }

  /**
   * Returns drupal asset library definition.
   */
  public function getDefinition() : array {
    return $this->library;
  }

  /**
   * Returns vite manifest.
   */
  public function getViteManifest() : Manifest {
    $manifestPath = $this->getManifestPath();
    $baseUrl = $this->getBaseUrl();
    return new Manifest($manifestPath, $baseUrl);
  }

  /**
   * Returns base url used in rewriting library for dist.
   */
  private function getBaseUrl() : string {
    $baseUrl = $this->getViteSetting('baseUrl');
    if (empty($baseUrl) || !is_string($baseUrl)) {
      $baseUrl = $this->getViteSettingFromLibraryConfig('baseUrl');
    }
    if (empty($baseUrl) || !is_string($baseUrl)) {
      $baseUrl = '/' . $this->getExtensionBasePath() . '/dist/';
    }
    return $baseUrl;
  }

  /**
   * Returns absolute vite manifest path.
   */
  private function getManifestPath() : string {
    $manifestPath = $this->getViteSetting('manifest');
    if (empty($manifestPath) || !is_string($manifestPath)) {
      $manifestPath = $this->getViteSettingFromLibraryConfig('manifest');
    }
    if (empty($manifestPath) || !is_string($manifestPath)) {
      $manifestPath = 'dist/manifest.json';
    }
    $basePath = $this->getExtensionBasePath();
    return DRUPAL_ROOT . '/' . $basePath . '/' . $manifestPath;
  }

  /**
   * Returns library extension (module/theme) base path.
   */
  private function getExtensionBasePath() : string {
    $themes = \Drupal::service('extension.list.theme');
    $modules = \Drupal::service('extension.list.module');
    if ($themes->exists($this->extension)) {
      return $themes->getPath($this->extension);
    }
    elseif ($modules->exists($this->extension)) {
      return $modules->getPath($this->extension);
    }
    throw new \Exception('Could not find library extension (module/theme) base path.');
  }

  /**
   * Checks if library should be managed by vite.
   */
  public function shouldBeManagedByVite() : bool {
    $enabledInSettings = $this->getViteSetting('enabled');
    if (!is_bool($enabledInSettings)) {
      $enabledInSettings = FALSE;
    }
    $enabledInLibraryDefinition = isset($this->library['vite']) && $this->library['vite'] !== FALSE && (!isset($this->library['vite']['enabled']) || $this->library['vite']['enabled'] === TRUE);
    return $enabledInSettings || $enabledInLibraryDefinition;
  }

  /**
   * Determines if vite dev server or dist build should serve library assets.
   */
  public function shouldUseDevServer() : bool {
    try {
      return \Drupal::httpClient()->request('GET', $this->getDevServerBaseUrl(), ['http_errors' => FALSE])->getStatusCode() === 404;
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

  /**
   * Returns base url of vite dev server for the library.
   */
  public function getDevServerBaseUrl() : string {
    $baseUrl = $this->getViteSetting('devServerUrl');
    if (!is_string($baseUrl) || !UrlHelper::isValid($baseUrl)) {
      $baseUrl = $this->getViteSettingFromLibraryConfig('devServerUrl');
    }
    if (!is_string($baseUrl) || !UrlHelper::isValid($baseUrl)) {
      $baseUrl = 'http://localhost:5173';
    }
    return $baseUrl;
  }

  /**
   * Returns vite setting for the library or NULL.
   */
  private function getViteSetting(string $setting) : mixed {
    $settings = Settings::get('vite', []);
    if (!is_array($settings)) {
      return NULL;
    }

    $value = NULL;

    // Global settings.
    if (!empty($settings[$setting])) {
      $value = $settings[$setting];
    }

    // Extension specific settings.
    if (!empty($settings['overrides'][$this->extension][$setting])) {
      $value = $settings['overrides'][$this->extension][$setting];
    }

    // Library specific settings.
    if (!empty($settings['overrides'][$this->extension . '/' . $this->libraryId][$setting])) {
      $value = $settings['overrides'][$this->extension . '/' . $this->libraryId][$setting];
    }

    return $value;
  }

  /**
   * Returns vite library config for the library or NULL.
   */
  private function getViteSettingFromLibraryConfig(string $setting) : mixed {
    if (!isset($this->library['vite']) || !is_array($this->library['vite'])) {
      return NULL;
    }
    $value = NULL;
    if (!empty($this->library['vite'][$setting])) {
      $value = $this->library['vite'][$setting];
    }
    return $value;
  }

}
